<?php

namespace Drupal\fox\Plugin\FoxCommand;

/**
 * USE fox command.
 *
 * @FoxCommand(
 *   id = "list",
 *   label = @Translation("List entities,bundles or fields list for entity,bundle. Usage: LIST [entity_id [bundle]]")
 * )
 */
class FoxCommandList extends FoxCommandBaseClass {

  /**
   * {@inheritdoc}
   */
  public function execute(array $params, array $variables): array {
    if (!empty($params)) {
      return $this->fieldDefinitions($params);
    }

    $header = [$this->t('ID'), $this->t('Title'), $this->t('Bundles')];

    $helper = $this->foxCommandsHelper();
    $all_entity_types = $helper->entityTypeManager()->getDefinitions();
    $content_entities = array_filter($all_entity_types, function ($entityType) {
      return $entityType->getGroup() == 'content';
    });

    $entity_type_bundle_Info = $helper->entityTypeBundleInfo();
    $data = [];
    foreach ($content_entities as $content_entity) {
      $id = $content_entity->id();
      $label = (string) $content_entity->getLabel();

      $bundle_data = '';
      $bundle_info = $entity_type_bundle_Info->getBundleInfo($id);
      if (!empty($bundle_info)) {
        $bundles = [];
        foreach ($bundle_info as $bundle_id => $bundle) {
          $bundle_label = (string) $bundle['label'];
          $bundles[] = sprintf('%s (%s)', $bundle_id, $bundle_label);
        }
        $bundle_data = implode(', ', $bundles);
      }
      $data[] = [$id, $label, $bundle_data];
    }

    return [
      'message' => [
        'header' => $header,
        'data' => $data,
      ],
    ];
  }

  /**
   * Get field definitions.
   *
   * @param array $params
   *   Entity type and bundle.
   *
   * @return array
   *   Fields definitions.
   */
  protected function fieldDefinitions(array $params): array {
    $entity_id = $params[0];
    $bundle = $params[1] ?? $entity_id;

    try {
      $definitions = $this->foxCommandsHelper()
        ->entityFieldManager()
        ->getFieldDefinitions($entity_id, $bundle);
    }
    catch (\Exception $e) {
      return [
        'error' => $e->getMessage(),
      ];
    }

    $header = [$this->t('ID'), $this->t('Name')];
    $data = [];
    foreach ($definitions as $field) {
      $data[] = [$field->getName(), $field->getLabel()];
    }

    return [
      'message' => [
        'header' => $header,
        'data' => $data,
      ],
    ];
  }

}
